/*******************************************************************************
File:			Main.c

				Main function library for Ethernet to IR gateway

Functions:	extern int main(void)
				SIGNAL(SIG_OVERFLOW0)

Revisions:	1.00	22/22/13	GND	Gary N. Dion
*******************************************************************************/

// OS headers
#include <avr/io.h>
#include <avr/interrupt.h>

// General purpose include files
#include "StdDefines.h"

// App required include files
#include "Main.h"
#include "Serial.h"

#define	WatchdogReset() asm("wdr")		// Macro substitution to kick the dog

// Local variables
volatile unsigned char IR_Cycles;		// A flag cleared in the Timer 1 ISR
unsigned short Freq_set = 138;				// 166 default is 40Khz

#define LOOPS (3)


/******************************************************************************/
extern int main(void)
/*******************************************************************************
* ABSTRACT:	Main program entry function.
*
* INPUT:		None
* RETURN:	int
*/
{	
	unsigned char temp;						// Just used for the loops counter
	unsigned char index;						// Used to index the burst sequence
	unsigned char nextchar;					// Holds the latest serial port character
	unsigned char spaces = 0;				// Spaces received during parsing
	unsigned char bursts = 26;				// Default bursts for pronto[] array
	unsigned char encoded = 0;				// Non-zero during an encoded character
	unsigned char decoding = 0;			// True while parsing the incomming msg
	unsigned short burst_val = 0;			// Value of the burst being decoded
	unsigned char burst_char_letter = 0;// Letter of burst being converted
	unsigned char pronto[100] = {0x00, 0x67, 0x00, 0x0d, 0x60, 0x18, 0x18, 0x18, 
									0x18, 0x18, 0x30, 0x18, 0x18, 0x18, 0x30, 0x18, 0x18,
									0x18, 0x18, 0x18,	0x30, 0x18, 0x18, 0x18, 0x18, 0x18,
									0x18, 0x18, 0x18, 0xff};  // default is SONY OFF.
	unsigned char delays[32] = {190, 187, 184, 180, 177, 174, 170, 167,
										 164, 160, 157, 154, 150, 147, 144, 140,
										 137, 134, 130, 127, 124, 120, 117, 114,
										 110, 107, 104, 100,  97,  94,  90,  86};
	
	// Initialize serial communication functions
	SerInit();

	//	Initialize the 16-bit Timer1 to clock at the XTAL freq. of 14.7456 MHz
	// ... with a Waveform Generation Mode of "6" (Fast PWM, 9-bit)
//	TCCR1A = (1<<COM1A1) | (1<<WGM11);	// COM1A1 turns on OC1A pin
	TCCR1A = (1<<WGM11);						// WGM bits set the mode
	TCCR1B = (1<<WGM12) | (1<<CS10);		// CS10 turns on counter
	TCNT1 = 0;									// Set first trip in 512 clocks
	TIMSK |= 1<<TOIE1;						// Enable Timer1 interrupt

	// Enable the watchdog timer
//	WDTCR	= (1<<WDCE) | (1<<WDE);	// Wake-up the watchdog register
//	WDTCR	= (1<<WDE) | 7;			// Enable and timeout around 2.1s

	// Enable interrupts
	sei();

	// Reset watchdog
	WatchdogReset();

	OCR1A = 262;								// This value makes a ~33% duty cycle
	TCCR1A |= (1<<COM1A1);				// COM1A1 turns on OC1A pin

	// PORT B - Just one pin, the OC1A
	// Bit/Pin 1 = Output of OC1A used for modulating the LED.
	DDRB = 2;
//while (1);

	// Initialization complete - system ready.  Run program loop indefinitely.
	while (TRUE)
	{
//		WatchdogReset();						// Reset watchdog
		nextchar = SerGetByte();			// Get latest received byte, if any
		if (nextchar == 0) continue;		// If no new byte, repeat

		if ((nextchar > 0x60) && (nextchar < 0x67))	// Lowercase a through f
		{
			nextchar -= 0x20;					// Convert to uppercase
		}
		
		if ((nextchar > 0x1F) && (nextchar < 0x47))	// Only range we care about
		{
//			SerSendByte(nextchar);			// If a valid byte, print (debug)

			if (encoded == 2)
			{
				if (nextchar == '0')  		// and next char is '0'
				{
					nextchar = ' ';			// a URL encoded space is %20
				}

				encoded = 0;					// Done, move along
			}
			
			if (encoded == 1)
			{	
				if (nextchar == '2')			// Decoding, and char is '2'
	 			{
					encoded = 2;				// Remember for next time
				}
				else
				{
					encoded = 0;				// Something went wrong
				}
				
			}
			
			if (nextchar == '%')				// Start of a URL encoded character
			{
				encoded = 1;					// So remember for next time
			}
			
		if (encoded == 0)			// Skip the rest if decoding a char
		{
			SerSendByte(nextchar);			// If a valid byte, print (debug)

			if (nextchar == '(')				// Start of burst pair description
			{
				decoding = TRUE;				// Yes, we're decoding now
				spaces = 0;						// So initialize spaces count
				burst_val = 0;					// And clear the character
				burst_char_letter = 3;		// And numeral to convert
				bursts = 0;
			}

			if (nextchar == ')')				// Transmit IR signal
			{
				if (pronto[1] < 0x57) 
				{
					Freq_set = 275;			// This results in 56.5khz
					OCR1A = 324;				// This value makes a ~33% duty cycle

				}
				else
				{
					if (pronto[1] < 0x60) pronto[1] = 0x60;
					if (pronto[1] > 0x7F) pronto[1] = 0x7F;
					Freq_set = delays[pronto[1] - 0x60];
					OCR1A = 262;				// This value makes a ~33% duty cycle

				}
				
				for (temp = 0; temp < LOOPS; temp++)
				{
					for (index = 4; index < bursts; index +=2)
					{
						TCCR1A |= (1<<COM1A1);
						IR_Cycles = pronto[index];
						while(IR_Cycles);
						TCCR1A &= ~(1<<COM1A1);
						IR_Cycles = pronto[index+1];
						while(IR_Cycles);
					}

					while(IR_Cycles);
					while(IR_Cycles);
				}
				
				decoding = FALSE;				// Done, stop decoding
			}
			
			if (decoding)						// If decoding a burst sequence
			{
				if (nextchar == ' ')			// Is a space, duh
				{
					spaces++;					// Increment spaces count
					burst_val = 0;				// And clear the character
					burst_char_letter = 3;	// Reset numeral to convert
				}
				
				if ((nextchar > 0x2F) && (nextchar < 0x3A)) // Is a numeral
				{
					nextchar -= 0x30;			// Convert character to number
				}
				
				if ((nextchar > 0x40) && (nextchar < 0x47)) // Is A through F
				{
					nextchar -= 0x37;			// Convert character to number
				}
								
				if (nextchar < 16) 			// If a valid 4-bit number (should be!)
				{
					// First, shift the nibble to the right location and sum
					burst_val += (nextchar<<(burst_char_letter * 4));
					
					if (burst_char_letter == 0)	// If this was the last nubble
					{
						if (burst_val > 0xFF) burst_val = 0xFF; // Limit val to 8-bit
						pronto[spaces] = burst_val;	// Save burst in array
						bursts++;				// And increment burst count
					}
					else burst_char_letter--;	// Else, get ready for next nibble
					
				}
				
			}
			
		}

		
		}
		else
		{
			decoding = FALSE;
			encoded = 0;
		}
		
	}

	return(1);

}		// END Main(...)


/******************************************************************************/
SIGNAL(TIMER1_OVF_vect)
/*******************************************************************************
* ABSTRACT:	This function handles the counter1 overflow interrupt.  This timer's
*				counter increments at 14.7456MHz.  And TOP is 0x01FF.
*				For the interrupt	to occur at 38 kHz, preload the counter with 124.
*				For the interrupt	to occur at 40 kHz, preload the counter with 144.
*				Note, there is no prescale for this counter it's incrementing on
*				every cycle, which means it's really difficult to get an -exact-
*				timing in here.  So these numbers may need to be changed by trial
*				and error.  Most IR receivers will tolerate 10% error, so don't
*				sweat it too much.
*				Trial and error numbers: 147 for 38khz, and 166 for 40khz.
*						157 for 39khz, 137 for 37khz, 
*
* PARAMETERS:	None.
*/
{
	//	Preload the counter to interrupt at 38 kHz
	TCNT1 = Freq_set;
	IR_Cycles--;

}		// End SIGNAL(TIMER0_OVF_vect)

